Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.Net.Sockets
Imports System.Threading
Imports Bdt.Shared.Logs
Imports Bdt.Shared.Service
Imports Bdt.Shared.Request
Imports Bdt.Shared.Response
#End Region

Namespace Bdt.Client.Sockets

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Passerelle entre le tunnel et le socket local, initie par le serveur tcp
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class Gateway
        Inherits LoggedObject

#Region " Constantes "
        ' La taille du buffer d'IO
        Public Const BUFFER_SIZE As Integer = 8192
        ' La dure minimale entre deux tests de l'tat de connexion
        Public Const STATE_POLLING_TIME As Integer = 50
        ' Le test de la connexion effective
        Public Const SOCKET_TEST_POLLING_TIME As Integer = 100
#End Region

#Region " Attributs "
        Protected m_client As TcpClient
        Protected m_stream As NetworkStream
        Protected m_mre As New ManualResetEvent(False)
        Protected m_tunnel As ITunnel
        Protected m_uid As Integer
        Protected m_address As String
        Protected m_port As Integer
        Protected m_cid As Integer
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur
        ''' </summary>
        ''' <param name="client">le socket client pour la communication locale</param>
        ''' <param name="tunnel">le tunnel pour la communication distante</param>
        ''' <param name="uid">le user-id</param>
        ''' <param name="address">l'adresse distante de connexion</param>
        ''' <param name="port">le port distant de connexion</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal client As TcpClient, ByVal tunnel As ITunnel, ByVal uid As Integer, ByVal address As String, ByVal port As Integer)
            m_client = client
            m_tunnel = tunnel
            m_uid = uid
            m_stream = client.GetStream
            m_address = address
            m_port = port

            Dim thr As New Thread(AddressOf CommunicationThread)
            thr.IsBackground = True
            thr.Start()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gestion des erreurs sur une rponse du tunnel
        ''' </summary>
        ''' <param name="response">la rponse du tunnel</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub HandleError(ByVal response As GenericResponse)
            HandleError(response.Message)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gestion des erreurs
        ''' </summary>
        ''' <param name="ex">l'exception  grer</param>
        ''' <param name="show">affichage du message d'erreur</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub HandleError(ByVal ex As Exception, Optional ByVal show As Boolean = True)
            HandleError(ex.Message, show)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gestion des erreurs
        ''' </summary>
        ''' <param name="message">le message  grer</param>
        ''' <param name="show">affichage du message d'erreur</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub HandleError(ByVal message As String, Optional ByVal show As Boolean = True)
            If show Then
                Log(message, ILogger.ESeverity.ERROR)
            End If
            m_mre.Set()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Gestion de l'tat de connexion
        ''' </summary>
        ''' <param name="response">la rponse du tunnel</param>
        ''' -----------------------------------------------------------------------------
        Protected Sub HandleState(ByVal response As GenericResponse)
            If Not response.Connected Then
                m_mre.Set()
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Traitement principal du thread
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Sub CommunicationThread()
            Dim buffer(BUFFER_SIZE) As Byte

            Dim response As ConnectResponse = m_tunnel.Connect(New ConnectRequest(m_uid, m_address, m_port))
            Log(response.Message, ILogger.ESeverity.DEBUG)
            If response.Success Then
                m_cid = response.Cid

                While (Not m_mre.WaitOne(STATE_POLLING_TIME, False))
                    ' Si des donnes sont prsentes sur le socket, envoi au tunnel
                    Dim isConnected As Boolean = False
                    Dim isDataAvailAble As Boolean = False

                    Try
                        isConnected = (Not (m_client.Client.Poll(SOCKET_TEST_POLLING_TIME, System.Net.Sockets.SelectMode.SelectRead) AndAlso m_client.Client.Available = 0))
                        isDataAvailAble = m_stream.DataAvailable()
                    Catch ex As Exception
                        HandleError(ex, False)
                    End Try

                    If isConnected Then
                        If isDataAvailAble Then
                            Dim count As Integer = 0
                            Try
                                count = m_stream.Read(buffer, 0, BUFFER_SIZE)
                            Catch ex As Exception
                                HandleError(ex)
                            End Try
                            If count > 0 Then
                                ' Des donnes sont prsentes en local, envoi
                                Dim transBuffer(count - 1) As Byte
                                Array.Copy(buffer, transBuffer, count)
                                Bdt.Shared.Runtime.Program.RandomXorEncoder(transBuffer, m_cid)
                                Dim writeResponse As GenericResponse = m_tunnel.Write(New WriteRequest(m_uid, m_cid, transBuffer))
                                transBuffer = Nothing
                                If writeResponse.Success Then
                                    HandleState(writeResponse)
                                Else
                                    HandleError(writeResponse)
                                End If
                            End If
                        End If


                        ' Si des donnes sont prsentes dans le tunnel, envoi au socket
                        Dim readResponse As ReadResponse = m_tunnel.Read(New GenericRequest(m_uid, m_cid))
                        If readResponse.Success Then
                            If readResponse.Connected AndAlso readResponse.DataAvailable Then
                                ' Puis criture sur le socket local
                                Bdt.Shared.Runtime.Program.RandomXorEncoder(readResponse.Data, m_cid)
                                Try
                                    m_stream.Write(readResponse.Data, 0, readResponse.Data.Length)
                                Catch ex As Exception
                                    HandleError(ex)
                                End Try
                            Else
                                HandleState(readResponse)
                            End If
                        Else
                            HandleError(readResponse)
                        End If
                    Else
                        ' Deconnexion
                        m_mre.Set()
                    End If

                End While
                Disconnect()
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Deconnexion
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Sub Disconnect()
            If Not IsNothing(m_client) Then
                m_stream.Close()
                m_client.Close()
                Dim response As GenericResponse = m_tunnel.Disconnect(New GenericRequest(m_uid, m_cid))
                Log(response.Message, ILogger.ESeverity.DEBUG)
                m_stream = Nothing
                m_client = Nothing
            End If
        End Sub

#End Region

    End Class

End Namespace
