Option Explicit On
Option Strict On

#Region " Inclusions "
Imports System.Net.Sockets
#End Region

Namespace Bdt.Client.Socks

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gestionnaire Socks v5
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class Socks5Handler
        Inherits GenericSocksHandler

#Region " Constantes "
        ' Mthodes d'authentication
        Public Const SOCKS5_NO_AUTHENTICATION_REQUIRED As Integer = 0
        Public Const SOCKS5_GSSAPI As Integer = 1 ' Non supporte
        Public Const SOCKS5_USERNAME_PASSWORD As Integer = 2 ' Non supporte
        Public Const SOCKS5_NO_ACCEPTABLE_METHODS As Integer = 255

        ' Commandes
        Public Const SOCKS5_CONNECT_COMMAND As Integer = 1
        Public Const SOCKS5_BIND_COMMAND As Integer = 2
        Public Const SOCKS5_UDP_ASSOCIATE_COMMAND As Integer = 3

        ' Types d'adresses
        Public Const SOCKS5_IPV4 As Integer = 1
        Public Const SOCKS5_DOMAIN As Integer = 3
        Public Const SOCKS5_IPV6 As Integer = 4

        ' Responses
        Public Const SOCKS5_OK As Integer = 0
        Public Const SOCKS5_KO As Integer = 1
        Public Const SOCKS5_REPLY_VERSION As Integer = 5

        Public Const REPLY_SIZE As Integer = 10  ' octets de rponse
#End Region

#Region " Attributs "
        Protected m_reply(REPLY_SIZE - 1) As Byte
        Protected m_stream As NetworkStream
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Le handler est-il adapt  la requte?
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides ReadOnly Property IsHandled() As Boolean
            Get
                ' Prparation pessimiste de la rponse
                m_reply(0) = SOCKS5_REPLY_VERSION
                m_reply(1) = SOCKS5_KO
                m_reply(2) = 0
                m_reply(3) = SOCKS5_IPV4
                Array.Clear(m_reply, 4, 6)

                If (Version <> 5) Then Return False

                Dim numMethods As Integer = Buffer(1)
                Dim methodAccepted As Boolean = False

                If ((numMethods <= 0) OrElse (numMethods <> Buffer.Length - 2)) Then
                    Log("Enumeration de mthode Socks5 malforme", [Shared].Logs.ILogger.ESeverity.WARN)
                    Return False
                End If

                Dim handshake(1) As Byte
                Dim i As Integer = 0
                While ((i < numMethods) AndAlso (Not methodAccepted))
                    methodAccepted = (Buffer(i + 2) = SOCKS5_NO_AUTHENTICATION_REQUIRED)
                    i += 1
                End While
                handshake(0) = 5 ' version

                If (Not methodAccepted) Then
                    ' Ecriture du handshake
                    Log("Aucune authentification Socks5 n'est supporte", [Shared].Logs.ILogger.ESeverity.WARN)
                    handshake(1) = SOCKS5_NO_ACCEPTABLE_METHODS
                    m_stream.Write(handshake, 0, handshake.Length)
                    Return False
                End If

                ' Ecriture du handshake
                handshake(1) = SOCKS5_NO_AUTHENTICATION_REQUIRED
                m_stream.Write(handshake, 0, handshake.Length)

                ' Lecture de la requete de connexion
                Dim request(BUFFER_SIZE) As Byte
                Dim size As Integer = m_stream.Read(request, 0, request.Length)

                Version = request(0)
                Command = request(1)
                Dim addressType As Integer = request(3)

                If (Version <> 5) Then
                    Log("Version Socks5 erronne", [Shared].Logs.ILogger.ESeverity.WARN)
                    Return False
                End If

                If (Command = SOCKS5_CONNECT_COMMAND) Then
                    If (addressType = SOCKS5_IPV4) Then
                        RemotePort = 256 * Convert.ToInt32(request(8)) + Convert.ToInt32(request(9))
                        Address = request(4) & "." & request(5) & "." & request(6) & "." & request(7)
                        ' Prparation de la rponse
                        m_reply(1) = SOCKS5_OK
                        Array.Copy(request, 4, m_reply, 4, 6)
                        Log("Requte Socks5 supporte", [Shared].Logs.ILogger.ESeverity.DEBUG)
                        Return True
                    ElseIf (addressType = SOCKS5_DOMAIN) Then
                        Dim length As Integer = request(4)
                        Address = New String(System.Text.Encoding.ASCII.GetChars(request), 5, length)
                        RemotePort = 256 * Convert.ToInt32(request(length + 5)) + Convert.ToInt32(request(length + 6))
                        ' Prparation de la rponse
                        m_reply(1) = SOCKS5_OK
                        Array.Clear(m_reply, 4, 6)
                        Log("Requte Socks5 supporte", [Shared].Logs.ILogger.ESeverity.DEBUG)
                        Return True
                    ElseIf (addressType = SOCKS5_IPV6) Then
                        Log("Adresses Socks5 IPV6 non supportes", [Shared].Logs.ILogger.ESeverity.WARN)
                    End If
                ElseIf (Command = SOCKS5_BIND_COMMAND) Then
                    Log("Requte Socks5 BIND non supporte", [Shared].Logs.ILogger.ESeverity.WARN)
                ElseIf (Command = SOCKS5_UDP_ASSOCIATE_COMMAND) Then
                    Log("Requte Socks5 UDP non supporte", [Shared].Logs.ILogger.ESeverity.WARN)
                End If

                Return False
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Les donnes de rponse
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides ReadOnly Property Reply() As Byte()
            Get
                Return m_reply
            End Get
        End Property
#End Region

#Region " Methodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur
        ''' </summary>
        ''' <param name="client">Le client TCP</param>
        ''' <param name="buffer">Les donnes de la requte</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal client As TcpClient, ByVal buffer() As Byte)
            MyBase.New(buffer)
            Version = buffer(0)
            Command = buffer(1)
            m_stream = client.GetStream
        End Sub
#End Region

    End Class
End Namespace