Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.Net.Sockets
Imports System.Threading
Imports Bdt.Shared.Service
Imports Bdt.Shared.Configuration
Imports Bdt.Shared.Request
Imports Bdt.Shared.Response
#End Region

Namespace Bdt.Server.Service

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Le tunnel, assure les services de l'interface ITunnel
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Class Tunnel
        Inherits MarshalByRefObject
        Implements ITunnel

#Region " Constantes "
        Public Const BUFFER_SIZE As Integer = 8192
        Public Const SERVER_TAG As String = "[serveur] "
        Public Const POLLING_TIME As Integer = 1000
        ' Le test de la connexion effective
        Public Const SOCKET_TEST_POLLING_TIME As Integer = 100

        Protected Const CONFIG_USER_TEMPLATE As String = "users/{0}@"
        Protected Const CONFIG_USER_ENABLED As String = "enabled"
        Protected Const CONFIG_USER_PASSWORD As String = "password"
#End Region

#Region " Attributs "
        Protected m_connections As New Hashtable
        Protected m_users As New Hashtable
        Protected m_timeout As Integer = 60 * 60
        Protected Shared m_mre As New ManualResetEvent(False)
        Protected Shared m_configuration As ConfigPackage
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' La configuration serveur (d'aprs le fichier xml + ligne de commande)
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Property Configuration() As ConfigPackage
            Get
                Return m_configuration
            End Get
            Set(ByVal value As ConfigPackage)
                m_configuration = value
            End Set
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub New()
            Dim thr As New Thread(AddressOf CheckerThread)
            thr.Start()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Generation d'id non encore prsent dans une table de hash
        ''' </summary>
        ''' <param name="hash">la table  analyser</param>
        ''' <returns>un entier unique</returns>
        ''' -----------------------------------------------------------------------------
        Protected Function GetNewId(ByVal hash As Hashtable) As Integer
            Dim rnd As New Random
            Dim key As Integer = rnd.Next(0, Integer.MaxValue)
            While hash.ContainsKey(key)
                key += 1
            End While
            Return key
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Generation d'un identifiant utilisateur unique
        ''' </summary>
        ''' <returns>un entier unique</returns>
        ''' -----------------------------------------------------------------------------
        Protected Function GetNewUid() As Integer
            Return GetNewId(m_users)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Generation d'un identifiant de connexion unique
        ''' </summary>
        ''' <returns>un entier unique</returns>
        ''' -----------------------------------------------------------------------------
        Protected Function GetNewCid() As Integer
            Return GetNewId(m_connections)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Destructeur
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub Finalize()
            DisableChecking()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Annule la vrification
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Shared Sub DisableChecking()
            m_mre.Set()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fixe une dure de vie au tunnel
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overrides Function InitializeLifetimeService() As Object
            Return Nothing
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Traitement principal de thread de vrification des connexions
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Sub CheckerThread()
            Dim test As Integer = 0
            While (Not m_mre.WaitOne(POLLING_TIME, False))
                Dim deletes As New Hashtable
                For Each cid As Integer In m_connections.Keys()
                    Dim connection As TunnelConnection = DirectCast(m_connections(cid), TunnelConnection)
                    If Not IsNothing(connection) Then
                        If Now.Subtract(connection.LastAccess).TotalSeconds > m_timeout Then
                            deletes.Add(cid, connection)
                        End If
                    End If
                Next
                For Each cid As Integer In deletes.Keys
                    Dim connection As TunnelConnection = DirectCast(m_connections(cid), TunnelConnection)
                    Try
                        connection.Stream.Flush()
                        connection.Stream.Close()
                        connection.TcpClient.Close()
                        connection.Stream = Nothing
                        connection.TcpClient = Nothing
                    Finally
                        ' Suppression d'une entre dans la table des connexions
                        m_connections.Remove(cid)
                    End Try
                Next
            End While
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Vrification de l'authentification d'un utilisateur associ  une requte
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <param name="response">la rponse  prparer</param>
        ''' <returns>True si l'utilisateur est authentifi</returns>
        ''' -----------------------------------------------------------------------------
        Public Function CheckUser(ByVal request As GenericRequest, ByVal response As GenericResponse) As Boolean
            Dim user As TunnelUser = DirectCast(m_users(request.Uid), TunnelUser)
            If IsNothing(user) Then
                response.Success = False
                response.Message = SERVER_TAG & "Le uid est introuvable"
            Else
                response.Success = True
                response.Message = String.Empty
            End If
            Return response.Success
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Vrification de la connexion associe  une requte
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <param name="response">la rponse  prparer</param>
        ''' <returns>true si la connexion est valide</returns>
        ''' -----------------------------------------------------------------------------
        Public Function CheckConnection(ByVal request As GenericRequest, ByVal response As GenericResponse) As TunnelConnection
            Dim connection As TunnelConnection = DirectCast(m_connections(request.Cid), TunnelConnection)
            If IsNothing(connection) Then
                response.Success = False
                response.Message = SERVER_TAG & "Le cid est introuvable"
                Return Nothing
            Else
                connection.LastAccess = Now
                Try
                    response.Connected = (Not (connection.TcpClient.Client.Poll(SOCKET_TEST_POLLING_TIME, System.Net.Sockets.SelectMode.SelectRead) AndAlso connection.TcpClient.Client.Available = 0))
                    response.DataAvailable = connection.TcpClient.Available > 0
                Catch ex As Exception
                    response.Connected = False
                    response.DataAvailable = False
                End Try
                response.Success = True
                response.Message = String.Empty
                Return connection
            End If
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Authentification d'un utilisateur
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <returns>une rponse AuthenticateResponse avec un uid</returns>
        ''' -----------------------------------------------------------------------------
        Public Function Authenticate(ByVal request As AuthenticateRequest) As AuthenticateResponse Implements ITunnel.Authenticate
            Dim enabled As Boolean = Configuration.ValueBool(String.Format(CONFIG_USER_TEMPLATE, request.Username) & CONFIG_USER_ENABLED, False)
            Dim password As String = Configuration.Value(String.Format(CONFIG_USER_TEMPLATE, request.Username) & CONFIG_USER_PASSWORD)
            If Not enabled Then
                Return New AuthenticateResponse(False, String.Format("{0}Authentification refuse pour {1}: le compte est inactif ou inexistant", SERVER_TAG, request.Username), -1)
            Else
                If password = request.Password Then
                    Dim user As New TunnelUser
                    user.Logon = Now
                    user.Uid = GetNewUid()
                    user.Username = request.Username
                    m_users.Add(user.Uid, user)
                    Return New AuthenticateResponse(True, String.Format("{0}Authentification accepte pour {1}", SERVER_TAG, request.Username), user.Uid)
                Else
                    Return New AuthenticateResponse(False, String.Format("{0}Authentification refuse pour {1}: le mot de passe est incorrect", SERVER_TAG, request.Username), -1)
                End If
            End If
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' La version du serveur
        ''' </summary>
        ''' <returns>La version de l'assembly</returns>
        ''' -----------------------------------------------------------------------------
        Public Function Version() As GenericResponse Implements ITunnel.Version
            With Me.GetType.Assembly.GetName
                Return New GenericResponse(True, SERVER_TAG & String.Format("{0} v{1}", .Name, .Version.ToString()))
            End With
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Etablissement d'une nouvelle connexion
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <returns>le cid de connexion</returns>
        ''' -----------------------------------------------------------------------------
        Public Function Connect(ByVal request As ConnectRequest) As ConnectResponse Implements ITunnel.Connect
            Dim response As New ConnectResponse

            If CheckUser(request, response) Then
                ' Cration d'une entre dans la table des connexions
                Dim connection As New TunnelConnection
                Try
                    connection.TcpClient = New TcpClient(request.Address, request.Port)
                    ' On obtient le socket protg par introspection
                    connection.Stream = connection.TcpClient.GetStream
                    connection.LastAccess = Now
                    connection.Uid = request.Uid
                    response.Connected = True
                    response.DataAvailable = connection.Stream.DataAvailable
                    Dim cid As Integer = GetNewCid()
                    m_connections.Add(cid, connection)

                    ' Ok
                    response.Success = True
                    response.Message = String.Format("{0}Connexion effectue  {1} ({2})", SERVER_TAG, connection.TcpClient.Client.RemoteEndPoint, request.Address)
                    response.Cid = cid
                Catch ex As Exception
                    response.Success = False
                    response.Message = String.Format("{0}Connexion refuse  {1}:{2} {3}", SERVER_TAG, request.Address, request.Port, ex.Message)
                    response.Cid = -1
                End Try
            End If

            Return response
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Dconnexion
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <returns>une reponse gnrique</returns>
        ''' -----------------------------------------------------------------------------
        Public Function Disconnect(ByVal request As GenericRequest) As GenericResponse Implements ITunnel.Disconnect
            Dim response As New GenericResponse

            If CheckUser(request, response) Then
                Dim connection As TunnelConnection = CheckConnection(request, response)
                If Not IsNothing(connection) Then
                    Try
                        response.Message = String.Format("{0}Dconnexion effectue de {1}", SERVER_TAG, connection.TcpClient.Client.RemoteEndPoint.ToString)

                        connection.Stream.Flush()
                        connection.Stream.Close()
                        connection.TcpClient.Close()
                        connection.Stream = Nothing
                        connection.TcpClient = Nothing

                        ' Ok
                        response.Connected = False
                        response.DataAvailable = False
                        response.Success = True

                        ' Suppression d'une entre dans la table des connexions
                        m_connections.Remove(request.Cid)

                    Catch ex As Exception
                        response.Success = False
                        response.Message = String.Format("{0}{1}", SERVER_TAG, ex.Message)
                    End Try
                End If
            End If

            Return response
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Lecture de donnes
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <returns>les donnes lues dans response.Data</returns>
        ''' -----------------------------------------------------------------------------
        Public Function Read(ByVal request As GenericRequest) As ReadResponse Implements ITunnel.Read
            Dim response As New ReadResponse

            If CheckUser(request, response) Then
                Dim connection As TunnelConnection = CheckConnection(request, response)
                If Not IsNothing(connection) Then
                    If response.Connected AndAlso response.DataAvailable Then
                        ' Donnes disponibles
                        Try
                            Dim buffer(BUFFER_SIZE) As Byte
                            Dim count As Integer = connection.Stream.Read(buffer, 0, BUFFER_SIZE)
                            If count > 0 Then
                                ReDim Preserve buffer(count - 1)
                                response.Success = True
                                response.Message = String.Empty
                                Bdt.Shared.Runtime.Program.RandomXorEncoder(buffer, request.Cid)
                                response.Data = buffer
                            Else
                                response.Success = False
                                response.Data = Nothing
                                response.Message = String.Format("{0}{1}", SERVER_TAG, "Dconnexion dtecte")
                            End If
                        Catch ex As Exception
                            response.Success = False
                            response.Data = Nothing
                            response.Message = String.Format("{0}{1}", SERVER_TAG, ex.Message)
                        End Try
                    Else
                        ' Pas de donnes disponibles
                        response.Success = True
                        response.Message = String.Empty
                        response.Data = New Byte() {}
                    End If
                End If
            End If

            Return response
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ecriture de donnes
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <returns>une rponse gnrique</returns>
        ''' -----------------------------------------------------------------------------
        Public Function Write(ByVal request As WriteRequest) As GenericResponse Implements ITunnel.Write
            Dim response As New GenericResponse

            If CheckUser(request, response) Then
                Dim connection As TunnelConnection = CheckConnection(request, response)
                If Not IsNothing(connection) Then
                    Try
                        Bdt.Shared.Runtime.Program.RandomXorEncoder(request.Data, request.Cid)
                        connection.Stream.Write(request.Data, 0, request.Data.Length)
                        response.Success = True
                        response.Message = String.Empty
                    Catch ex As Exception
                        response.Success = False
                        response.Message = String.Format("{0}{1}", SERVER_TAG, ex.Message)
                    End Try
                End If
            End If

            Return response
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne l'tat d'une connexion
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <returns>Une rponse gnrique</returns>
        ''' -----------------------------------------------------------------------------
        Public Function State(ByVal request As GenericRequest) As GenericResponse Implements ITunnel.State
            Dim response As New GenericResponse

            If CheckUser(request, response) Then
                CheckConnection(request, response)
            End If

            Return response
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne des informations sur les connexions tablies
        ''' </summary>
        ''' <param name="request">la requte</param>
        ''' <returns>la liste des connexions</returns>
        ''' -----------------------------------------------------------------------------
        Public Function Connections(ByVal request As GenericRequest) As ConnectionsResponse Implements ITunnel.Connections
            Dim response As New ConnectionsResponse

            If CheckUser(request, response) Then
                Try
                    Dim result As New ArrayList
                    For Each cid As Integer In m_connections.Keys
                        Dim connection As TunnelConnection = DirectCast(m_connections(cid), TunnelConnection)
                        If Not IsNothing(connection) Then
                            Dim user As TunnelUser = DirectCast(m_users(connection.Uid), TunnelUser)
                            result.Add(New Connection(user.Username, cid, connection.TcpClient.Client.RemoteEndPoint.ToString, connection.LastAccess, connection.Stream.DataAvailable))
                        End If
                    Next
                    response.Success = True
                    response.Message = String.Empty
                    response.Connections = DirectCast(result.ToArray(GetType(Connection)), Connection())
                Catch ex As Exception
                    response.Success = False
                    response.Message = String.Format("{0}{1}", SERVER_TAG, ex.Message)
                    response.Connections = Nothing
                End Try
            End If

            Return response
        End Function
#End Region

    End Class

End Namespace

