Option Explicit On 
Option Strict On

Namespace Bdt.Shared.Configuration

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Regroupe un ensemble de sources de configurations priorises. Permet de rechercher des 
    ''' valeurs suivant un code. 
    ''' Les sources peuvent tre diffrentes: base de donne, fichier de configuration,
    ''' ligne de commande, etc
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public NotInheritable Class ConfigPackage

#Region " Evenements "
        Public Event ReadString(ByVal sender As ConfigPackage, ByRef value As String)
        Public Event ReadInt(ByVal sender As ConfigPackage, ByRef value As Integer)
        Public Event ReadBool(ByVal sender As ConfigPackage, ByRef value As Boolean)
#End Region

#Region " Attributs "
        'Les sources sont tries par priorit grce au compareTo de SourceConfiguration (IComparable)
        Protected _Sources As New SortedList
#End Region

#Region " Proprits "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne la valeur d'un lment suivant son code
        ''' </summary>
        ''' <param name="code">le code de l'lment</param>
        ''' <param name="defaultValue">la valeur par dfaut si l'lment est introuvable</param>
        ''' <returns>La valeur de l'lment s'il existe ou defaultValue sinon</returns>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property Value(ByVal code As String, Optional ByVal defaultValue As String = Nothing) As String
            Get
                For Each source As BaseConfig In _Sources.Values
                    Dim result As String = source.Value(code, Nothing)
                    If Not IsNothing(result) Then
                        RaiseEvent ReadString(Me, result)
                        Return result
                    End If
                Next
                Return defaultValue
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fixe/Retourne la valeur entire d'un lment suivant son code
        ''' </summary>
        ''' <param name="code">le code de l'lment</param>
        ''' <param name="defaultValue">la valeur par dfaut si l'lment est introuvable</param>
        ''' <returns>La valeur de l'lment s'il existe et s'il reprsente un entier ou defaultValue sinon</returns>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property ValueInt(ByVal code As String, Optional ByVal defaultValue As Integer = 0) As Integer
            Get
                Try
                    Dim result As Integer = Integer.Parse(Value(code, defaultValue.ToString()))
                    RaiseEvent ReadInt(Me, result)
                    Return result
                Catch ex As Exception
                    Return defaultValue
                End Try
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fixe/Retourne la valeur boolenne d'un lment suivant son code
        ''' </summary>
        ''' <param name="code">le code de l'lment</param>
        ''' <param name="defaultValue">la valeur par dfaut si l'lment est introuvable</param>
        ''' <returns>La valeur de l'lment s'il existe et s'il reprsente un boolen (true/false) ou defaultValue sinon</returns>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property ValueBool(ByVal code As String, Optional ByVal defaultValue As Boolean = False) As Boolean
            Get
                Try
                    Dim result As Boolean = Boolean.Parse(Value(code, defaultValue.ToString()))
                    RaiseEvent ReadBool(Me, result)
                    Return result
                Catch ex As Exception
                    Return defaultValue
                End Try
            End Get
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ajoute une source  ce contexte de configuration
        ''' </summary>
        ''' <param name="source">la source  ajouter, qui sera classe par SourceConfiguration.Priority()</param>
        ''' -----------------------------------------------------------------------------
        Public Sub AddSource(ByVal source As BaseConfig)
            _Sources.Add(source, source)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Enleve une source de ce contexte de configuration
        ''' </summary>
        ''' <param name="source">la source  supprimer</param>
        ''' -----------------------------------------------------------------------------
        Public Sub RemoveSource(ByVal source As BaseConfig)
            _Sources.Remove(source)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Force un rechargement de toutes les sources de donnes lies
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Sub RehashAll()
            For Each source As BaseConfig In _Sources.Values
                source.Rehash()
            Next
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Concatne tous les lments depuis toutes les sources
        ''' </summary>
        ''' <returns>le format de chaque ligne est classe,priorit,code,valeur</returns>
        ''' -----------------------------------------------------------------------------
        Public Overrides Function ToString() As String
            ToString = String.Empty
            For Each source As BaseConfig In _Sources.Values
                ToString += source.ToString
            Next
        End Function
#End Region

    End Class

End Namespace
