Option Explicit On 
Option Strict On

#Region " Inclusions "
Imports System.IO
Imports System.ComponentModel
Imports Bdt.Shared.Configuration
#End Region

Namespace Bdt.Shared.Logs

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Classe de base pour la gnration d'un log 
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    <Serializable(), TypeConverter(GetType(ExpandableObjectConverter))> _
    Public MustInherit Class BaseLogger
        Implements ILogger

#Region " Constantes "
        Public Const DEFAULT_DATE_FORMAT As String = "dd/MM/yyyy HH:mm:ss"
        Public Const CONFIG_ENABLED As String = "enabled"
        Public Const CONFIG_FILTER As String = "filter"
        Public Const CONFIG_DATE_FORMAT As String = "dateformat"
        Public Const CONFIG_STRING_FORMAT As String = "stringformat"
        Public Const CONFIG_TAG_START As String = "{"
        Public Const CONFIG_TAG_END As String = "}"
#End Region

#Region " Enumerations "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ces tags permettent de construire une chaine de sortie personnalise
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Enum ETags
            TIMESTAMP = 0
            SEVERITY = 1
            TYPE = 2
            MESSAGE = 3
        End Enum
#End Region

#Region " Attributs "
        Protected m_writer As TextWriter = Nothing
        Protected m_stringFormat As String = CONFIG_TAG_START & ETags.TIMESTAMP & CONFIG_TAG_END & " " & CONFIG_TAG_START & ETags.SEVERITY & CONFIG_TAG_END & " [" & CONFIG_TAG_START & ETags.TYPE & CONFIG_TAG_END & "] " & CONFIG_TAG_START & ETags.MESSAGE & CONFIG_TAG_END
        Protected m_dateFormat As String = DEFAULT_DATE_FORMAT
        Protected m_filter As ILogger.ESeverity = ILogger.ESeverity.DEBUG
        Protected m_enabled As Boolean = True
#End Region

#Region " Proprietes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fixe/retourne l'tat d'activation du log
        ''' </summary>
        ''' <returns>l'tat d'activation du log</returns>
        ''' -----------------------------------------------------------------------------
        Public Property Enabled() As Boolean
            Get
                Return m_enabled
            End Get
            Set(ByVal Value As Boolean)
                m_enabled = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fixe/retourne la chaine personnalise pour la sortie des logs.
        ''' Utiliser les constantes ETags entre accolades {}
        ''' </summary>
        ''' <returns>la chaine personnalise pour la sortie des logs</returns>
        ''' -----------------------------------------------------------------------------
        Public Property StringFormat() As String
            Get
                Return m_stringFormat
            End Get
            Set(ByVal Value As String)
                m_stringFormat = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne/Fixe le format des dates de timestamp
        ''' </summary>
        ''' <returns>le format des dates de timestamp</returns>
        ''' -----------------------------------------------------------------------------
        Public Property DateFormat() As String
            Get
                Return m_dateFormat
            End Get
            Set(ByVal Value As String)
                m_dateFormat = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Retourne/fixe le niveau de filtrage pour la sortie des logs
        ''' </summary>
        ''' <returns>le niveau de filtrage pour la sortie des logs</returns>
        ''' -----------------------------------------------------------------------------
        Public Property Filter() As ILogger.ESeverity
            Get
                Return m_filter
            End Get
            Set(ByVal Value As ILogger.ESeverity)
                m_filter = Value
            End Set
        End Property
#End Region

#Region " Mthodes "
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur pour un log  partir des donnes fournies dans une configuration
        ''' </summary>
        ''' <param name="writer">le textwriter pour la sortie des logs</param>
        ''' <param name="prefix">le prefixe dans la configuration ex: application/log</param>
        ''' <param name="config">la configuration pour la lecture des parametres</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal writer As TextWriter, ByVal prefix As String, ByVal config As ConfigPackage)
            m_writer = writer

            m_enabled = config.ValueBool(prefix & BaseConfig.SOURCE_ITEM_ATTRIBUTE & CONFIG_ENABLED, m_enabled)
            m_filter = DirectCast(ILogger.ESeverity.Parse(GetType(ILogger.ESeverity), config.Value(prefix & BaseConfig.SOURCE_ITEM_ATTRIBUTE & CONFIG_FILTER, m_filter.ToString)), ILogger.ESeverity)
            m_dateFormat = config.Value(prefix & BaseConfig.SOURCE_ITEM_ATTRIBUTE & CONFIG_DATE_FORMAT, m_dateFormat)
            m_stringFormat = config.Value(prefix & BaseConfig.SOURCE_ITEM_ATTRIBUTE & CONFIG_STRING_FORMAT, m_stringFormat)

            ' Remplacement des id chaines de ETags en leur quivalent integer
            For Each tag As ETags In ETags.GetValues(GetType(ETags))
                m_stringFormat = Replace(m_stringFormat, tag.ToString, CStr(CInt(tag)))
            Next
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructeur pour un log
        ''' </summary>
        ''' <param name="writer">le textwriter pour la sortie des logs</param>
        ''' <param name="dateFormat">le format des dates de timestamp</param>
        ''' <param name="filter">le niveau de filtrage pour la sortie des logs</param>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal writer As TextWriter, Optional ByVal dateFormat As String = DEFAULT_DATE_FORMAT, Optional ByVal filter As ILogger.ESeverity = ILogger.ESeverity.DEBUG)
            m_writer = writer
            m_dateFormat = dateFormat
            m_filter = filter
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Ecriture d'une entre de log. Ne sera pas prise en compte si le log est inactif
        ''' ou si le filtre l'impose
        ''' </summary>
        ''' <param name="sender">l'emetteur</param>
        ''' <param name="message">le message  logger</param>
        ''' <param name="severity">la svrit</param>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub Log(ByVal sender As Object, ByVal message As String, ByVal severity As ILogger.ESeverity) Implements ILogger.Log
            If (m_enabled) AndAlso (severity >= m_filter) AndAlso (Not IsNothing(m_writer)) Then
                m_writer.WriteLine(String.Format(m_stringFormat, Now.ToString(m_dateFormat), severity, sender.GetType.Name, message))
                m_writer.Flush()
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Fermeture du logger
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub Close() Implements ILogger.Close
            If Not IsNothing(m_writer) Then
                Try
                    m_writer.Close()
                Catch ex As Exception
                End Try
                m_writer = Nothing
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Destruction d'un log
        ''' </summary>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub Finalize()
            Close()
            MyBase.Finalize()
        End Sub
#End Region

    End Class

End Namespace

